%bcond html_docs 1

Name:           gn
%global commit 88e8054aff7bd0cb2295c7d9361d2be0b7355f27
%global access 20240301
%global shortcommit 88e8054aff7b
%global position 2154
Version:        %{position}^%{access}git%{shortcommit}
Release:    1%{?dist}
Summary:        Meta-build system that generates build files for Ninja

License:        BSD-3-Clause AND Unicode AND ICU
URL:            https://gn.googlesource.com/gn
Source0:        %{url}/+archive/%{commit}.tar.gz#/gn-%{shortcommit}.tar.gz
Source1:        last_commit_position.h
Source2:        update-version

Patch:          gn-03d10f1657b4-no-O3.patch

ExcludeArch:    %{ix86}

BuildRequires:  python3-devel
BuildRequires:  ninja-build
BuildRequires:  gcc-c++

BuildRequires:  emacs-common

%if %{with html_docs}
BuildRequires:  pandoc
BuildRequires:  parallel
%endif
BuildRequires:  help2man

Requires:       vim-filesystem
Requires:       python3
Provides:       vim-gn = %{version}-%{release}

Requires:       emacs-filesystem >= %{_emacs_version}
Provides:       emacs-gn = %{version}-%{release}

Provides:       bundled(icu) = 60

%description
GN is a meta-build system that generates build files for Ninja.


%package doc
Summary:        Documentation for GN
BuildArch:      noarch

%description doc
The gn-doc package contains detailed documentation for GN.


%prep
%autosetup -c -n gn-%{commit} -p1

mkdir -p ./out
cp -vp '%{SOURCE1}' ./out

cp -vp misc/vim/README.md README-vim.md

%py3_shebang_fix .


%build
AR='gcc-ar'; export AR
%{python3} build/gen.py \
    --allow-warnings \
    --no-last-commit-position \
    --no-strip \
    --no-static-libstdc++
ninja -j %{_smp_build_ncpus} -C out -v

%if %{with html_docs}
find . -type f -name '*.md' | parallel -v pandoc -o '{.}.html' '{}'
%endif

help2man \
    --name='%{summary}' \
    --version-string="gn $(./out/gn --version)" \
    --no-info \
    ./out/gn |
  tr -d '\302\240' |
  sed -r -e 's/(^[[:alnum:]_]+:)/.TP\n.B \1\n/' \
      -e 's/\[([^]]+)\]/\\fI[\1]\\fR/g' > out/gn.1


%install
install -t '%{buildroot}%{_bindir}' -D -p out/gn

install -d '%{buildroot}%{_datadir}/vim/vimfiles'
cp -vrp misc/vim/* '%{buildroot}%{_datadir}/vim/vimfiles'
find '%{buildroot}%{_datadir}/vim/vimfiles' \
    -type f -name 'README.*' -print -delete
%py_byte_compile %{python3} '%{buildroot}%{_datadir}/vim/vimfiles/gn-format.py'

install -t '%{buildroot}%{_emacs_sitestartdir}' -D -p -m 0644 misc/emacs/*.el

install -t '%{buildroot}%{_mandir}/man1' -D -m 0644 -p out/gn.1


%check
out/gn_unittests

grep -E '^#define[[:blank:]]+LAST_COMMIT_POSITION_NUM[[:blank:]]+'\
'%{position}[[:blank:]]*' \
    'out/last_commit_position.h' >/dev/null
grep -E '^#define[[:blank:]]+LAST_COMMIT_POSITION[[:blank:]]+'\
'"%{position} \(%{shortcommit}\)"[[:blank:]]*' \
    'out/last_commit_position.h' >/dev/null


%files
%license LICENSE
%{_bindir}/gn

%{_mandir}/man1/gn.1*

%{_datadir}/vim/vimfiles/gn-format.py
%{_datadir}/vim/vimfiles/autoload/gn.vim
%{_datadir}/vim/vimfiles/ftdetect/gnfiletype.vim
%{_datadir}/vim/vimfiles/ftplugin/gn.vim
%{_datadir}/vim/vimfiles/syntax/gn.vim

%{_emacs_sitestartdir}/gn-mode.el


%files doc
%license LICENSE src/base/third_party/icu/README.chromium
%doc AUTHORS
%doc OWNERS
%doc README*.md
%if %{with html_docs}
%doc README*.html
%endif
%doc docs/
%doc examples/
%doc infra/
%doc tools/


%changelog
* Tue Jul 09 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 2154^20240301git88e8054aff7b-1
- initial build
